********************************************************************************
*                                                                              *
*  2DHF version 5-2005                                                         *
*  Copyright (C) 1996  Jacek Kobus, Leif Laaksonen, Dage Sundholm              *
*                                                                              *
*  This software may be used and distributed according to the terms            *
*  of the GNU General Public License, see README and COPYING.                  *
*                                                                              *
********************************************************************************
c ### inihf ###  

c     This routine initializes molecular orbitals as linear combinations
c     of Hartree-Fock functions (taken from the RHF program) on centres
c     A and B.  In the case of HF or HFS calculations Coulomb (exchange)
c     potentials are approximated as a linear combination of
c     Thomas-Fermi (1/r) potentials of the two centres. If method OED is
c     chosen the potential functions are set to zero.

      subroutine inihf (psi,pot,excp,f2,f4,wgt2,wk0)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      character*8 atom1,term1,atom2,term2
      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),f2(*),f4(*),wgt2(*),wk0(*)
      dimension rhf1(9999),phf1(20,9999),ehf1(20),qc1(20),
     &     nhf1(20),lhf1(20)
      dimension rhf2(9999),phf2(20,9999),ehf2(20),qc2(20),
     &     nhf2(20),lhf2(20)
      dimension phf1t(9999),phf2t(9999)
      parameter (iord=3)

c     Initialization of molecular orbitals 

      print *,'... initializing molecular orbitals from Hartree-Fock',
     &          ' functions ...'
      
      ilabel=0
      idebug1=1
      idebug2=0
      idebug3=0

c     read HF functions for centre A
      ouf2dhf1=51
      open(ouf2dhf1,form='formatted', status='old')
      
      read(ouf2dhf1,*) atom1,term1,zhf1,nwf1,mxmax1

      read(ouf2dhf1,'(2x,20i5)') (nhf1(i),i=1,nwf1)
      read(ouf2dhf1,'(2x,20i5)') (lhf1(i),i=1,nwf1)
      read(ouf2dhf1,'(3x,20f5.0)') (qc1(i),i=1,nwf1)
      read(ouf2dhf1,'(24x,20d24.16)') (ehf1(i),i=1,nwf1)
      do j=1,mxmax1
         read(ouf2dhf1,'(20d24.16)') rhf1(j),(phf1(i,j),i=1,nwf1)
      enddo

      if (idebug1.ne.0) then
         write(*,'(" Orbitals on centre Z1 (Z=",f4.0,"):")') 
     &        zhf1 
         write(*,'(13x,"n",4x,"l",9x,"e")') 
         do i=1,nwf1
            write(*,'(4x,3i5,d16.6)') i,nhf1(i),lhf1(i), ehf1(i)
         enddo

      endif
      
      if (idebug3.ne.0) then
         do j=1,mxmax1
            write(*,'(20d24.16)') rhf1(j),(phf1(i,j),i=1,nwf1)
         enddo
      endif

c     read HF functions for centre B
      ouf2dhf2=52
      open(ouf2dhf2,form='formatted', status='old')
      
      read(ouf2dhf2,*) atom2,term2,zhf2,nwf2,mxmax2
      read(ouf2dhf2,'(2x,20i5)') (nhf2(i),i=1,nwf2)
      read(ouf2dhf2,'(2x,20i5)') (lhf2(i),i=1,nwf2)
      read(ouf2dhf2,'(3x,20f5.0)') (qc2(i),i=1,nwf2)
      read(ouf2dhf2,'(24x,20d24.16)') (ehf2(i),i=1,nwf2)
      do j=1,mxmax2
         read(ouf2dhf2,'(20d24.16)') rhf2(j),(phf2(i,j),i=1,nwf2)
      enddo
      
      if (idebug1.ne.0) then
         write(*,*)
         write(*,'(" Orbitals on centre Z2 (Z=",f4.0,"):")') 
     &        zhf2
         write(*,'(13x,"n",4x,"l",9x,"e")') 
         do i=1,nwf2
            write(*,'(4x,3i5,d16.6)') i,nhf2(i),lhf2(i), ehf2(i)
         enddo

      endif
      
      if (idebug3.ne.0) then
         do j=1,mxmax2
            write(*,'(20d24.16)') rhf2(j),(phf2(i,j),i=1,nwf2)
         enddo
      endif

c     loop over orbitals

      do iorb=1,norb

         ishift=i1b(iorb)-1
         n1=mgx(1,iorb)
         l1=mgx(2,iorb)
         m1=mgx(3,iorb)

c        normalization factor for spherical harmonics

         shn1=(-1.d0)**dble(m1)/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
         if (m1.eq.0) shn1=1.d0/sqrt(4.d0*pii)*
     &        sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))

         if (co1(iorb).ne.0.d0) then
            ihf1=0
            do i=1,nwf1
               if (nhf1(i).eq.n1.and.lhf1(i).eq.l1) ihf1=i
            enddo
            if (ihf1.eq.0) then
               print *,"inihf: no proper atomic orbital for",
     &              " centre Z1 found"
               stop 'inihf'
            endif
            
            do j=1,mxmax1
               phf1t(j)=phf1(ihf1,j)
            enddo
         endif
         
         n2=mgx(4,iorb)
         l2=mgx(5,iorb)
         m2=mgx(6,iorb)

c        normalization factor for spherical harmonics

         shn2=(-1.d0)**dble(m2)/sqrt(4.d0*pii)*
     &        sqrt((2*l2+1)*factor(l2-m2)/factor(l2+m2))

         if (co2(iorb).ne.0.d0) then
            ihf2=0
            do i=1,nwf2
               if (nhf2(i).eq.n2.and.lhf2(i).eq.l2) ihf2=i
            enddo
            if (ihf2.eq.0) then
               print *,"inihf: no proper atomic orbital for",
     &              " centre Z2 found"
               stop 'inihf'
            endif
            
            do j=1,mxmax2
               phf2t(j)=phf2(ihf2,j)
            enddo
         endif

         if (idebug2.ne.0) then
            print *,'inihf: n1,l1,m1,shn1,ihf1',n1,l1,m1,shn1,ihf1
            print *,'inihf: n2,l2,m2,shn2,ihf2',n2,l2,m2,shn2,ihf2
         endif

c        loop over grid points

         psi1prev=0.d0
         psi2prev=0.d0
         do imu=1,mxnmu
            inioff=(imu-1)*nni
            do in=1,nni
               igp=ishift+inioff+in

c              for each grid point, e.i. for (vmu(imu),vni(ini)) 
c              determine its distance |_r1| and |_r2| from the nuclei A 
c              and B and cosine of the polar angles costh1 and costh2 
c              between z axis and the vectors _r1 and _r2

               psi1=0.d0
               psi2=0.d0
               
               rr=(r/2.d0)*sqrt(vxisq(imu)+vetasq(in)-1.d0)		
               z=(r/2.d0)*vxi(imu)*veta(in)
               r1t=(r/2.d0)*(vxi(imu)+veta(in))
               r2t=(r/2.d0)*(vxi(imu)-veta(in))
               
c              calculate radial part of the hydrogenic orbital centered 
c              on both the nuclei

               if (r1t.lt.precis) then
                  costh1=0.d0
               else   
                  costh1=(z+r/2.d0)/r1t
               endif
c              
               if (r2t.lt.precis) then
                  costh2=0.d0
               else   
                  costh2=-(z-r/2.d0)/r2t	
               endif
              
c              calculate radial part of the HF orbital taken from (R)HF
c              program as P=R*r

               if     (co1(iorb).ne.0.d0) then
                  if (r1t.ge.rhf1(mxmax1)) then
                     psi1=0.d0
                  elseif (r1t.ge.precis) then
                     psi1=flp(iord,mxmax1,rhf1,phf1t,r1t)/(r1t)*
     &                    shn1*plegendg(l1,m1,costh1)
                     psi1prev=psi1
                  else
                     psi1=psi1prev
                  endif
               endif
               
               if (co2(iorb).ne.0.d0) then	    
                  if (r2t.ge.rhf2(mxmax2)) then
                     psi2=0.d0
                  elseif (r2t.ge.precis) then
                     psi2=flp(iord,mxmax2,rhf2,phf2t,r2t)/(r2t)*
     &                    shn2*plegendg(l2,m2,costh2)
                     psi2prev=psi2
                  else
                     psi2=psi2prev
                  endif
               endif
00099          continue
               psi(igp)=co1(iorb)*psi1+co2(iorb)*psi2
            enddo
         enddo
         if (idebug1.ne.0) then
            if (ilabel.eq.0) then
               ilabel=1
               write (*,*) 
               write (*,*) 'Normalization of LCAOs'
            endif
            call norm94 (iorb,psi,f4,wgt2,wk0,xnorm)
            write (*,1115) iorn(iorb),bond(iorb),gut(iorb),xnorm
 1115       format(i4,1x,a8,a1,3x,d22.16,2d16.2)
         endif
      enddo

c  initialize Coulomb and exchange potentials
      
      call tfpot(psi,pot,excp,f2,f4,wk0)	
      return
      end
      

